﻿using Battleship.CollectionGenericObjects;
using Battleship.Drawings;

namespace Battleship;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormWarshipCollection : Form
{
    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Конструктор 
    /// </summary>
    public FormWarshipCollection()
    {
        InitializeComponent();
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        switch (comboBoxSelectionCompany.Text)
        {
            case "Хранилище":
                _company = new WarshipSharingService(pictureBox.Width, pictureBox.Height, new MassiveGenericObjects<DrawingWarship>());
                break;
        }
    }

    /// <summary>
    /// Создание объекта
    /// </summary>
    /// <param name="type"></param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }
        Random random = new();
        DrawingWarship drawingWarship;
        switch (type)
        {
            case nameof(DrawingWarship):
                drawingWarship = new DrawingWarship(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawingBattleship):
                drawingWarship = new DrawingBattleship(random.Next(100, 300), random.Next(1000, 3000),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        if (_company + drawingWarship)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
    /// Получение цвета
    /// </summary>
    /// <param name="random">Генератор случайных чисел</param>
    /// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }

    /// <summary>
    /// Добавление линкора
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonAddBattleship_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawingBattleship));
    }

    /// <summary>
    /// Добавление военного корабля
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonAddWarship_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawingWarship));
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveCar_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        if (_company - pos)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    /// <summary>
    /// Передача на тесты
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawingWarship? warship = null;
        int counter = 100;
        while (warship == null) 
        {
            warship = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        FormBattleship form = new FormBattleship();
        form.SetWarship = warship;
        form.ShowDialog();

    }

    /// <summary>
    /// Обновление
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }
}